#!/usr/bin/perl -w

############################################################
##                                                        ##
##           MRCA_COUNT.PL                                ##
##                                                        ##
##                       Programmed by Yoshihito Niimura  ##
##                       Last Modified on '06.8.8         ##
##                                                        ##
############################################################

if ($#ARGV < 2)
{
	print "\n";
	print "This program counts the numbre of genens at each ancestral node.\n";
	print "$0  *.BRANCHOUT  SPECIES_TREE_DATA  GENE_LIST  [-b THRESHOLD_BOOTSTRAP(n)]\n\n";
	print "SPECIES_TREE_DATA should be : CLADE1 CLADE2 OUTGROUP\n";
	print "e.g. Human~Mouse~Dog~Cow Mono 0\n";
	print "     Human~Mouse Dog~Cow Mono\n";
	print "     Human Mouse Dog~Cow~Mono\n";
	print "     Dog Cow Human~Mouse~Mono\n";
	print "GENE_LIST should be : GENE_NAME SPECIES (Outgroup genes must be excluded!!)\n";
	print "e.g. HsOR1.1.1 Human\n";
	print "     MmOR1.2.3 Mouse\n";
	print "     HsOR1.1.3 Human~Mouse~Dog~Cow\n";
	print "     .....\n";
	print "     # A subclade can be substituted by a representative gene.\n";
	print "THRESHOLD_BOOTSTRAP : Default = 350 (70% in 500 bootstrapping)\n\n";
	print "OUTPUT:\n";
	print "  #_of_Genes_in_MRCA  Gene_Gain_in_Branch1 / Gene_Loss_in_Branch1  Gene_Gain_in_Branch2 / Gene_Loss_in_Branch2\n\n";
	exit;
}

open(FILE,$ARGV[1]) || die "Cannot open $ARGV[1]\n";
while(<FILE>)
{
	@fld = split;
	@spe1 = split(/~/, $fld[0]);
	@spe2 = split(/~/, $fld[1]);
	@spe_list = (@spe1, @spe2);
	last;
}
foreach $spe(@spe_list)
{
	$num{$spe} = 0;
}

open(FILE,$ARGV[2]) || die "Cannot open $ARGV[2]\n";
while(<FILE>)
{
	@fld = split;
	$spe{$fld[0]} = $fld[1];
	$num{$spe{$fld[0]}}++;
}
close(FILE);
$boot = 350;
if ($ARGV[3] && $ARGV[3] eq "-b")
{
	$boot = $ARGV[4];
}

$l = 0;
open(FILE,$ARGV[1]) || die "Cannot open $ARGV[1]\n";
while(<FILE>)
{
	chomp;
	$speline = $_;
	@fld2 = split;
	$clade1 = $fld2[0];
	$clade2 = $fld2[1];
	$outgp  = $fld2[2];
	$n = 0;
	@checkline = @sorted_line = ();
	%member_num = ();
	foreach $gene (keys(%spe))
	{
		$flag{$gene} = 0;
	}
	open(FILE2,$ARGV[0]) || die "Cannot open $ARGV[0]\n";
	while(<FILE2>)
	{
		chomp;
		$line = $_;
		@fld = split;
		if ($fld[1] eq "O" || $fld[1] eq "r")
		{
			next;
		}
		if ($fld[1] eq "R" || $fld[4] >= $boot)
		{
			$flag1 = $flag2 = $flag3 = 0;
			for ($i = 6; $i <= $#fld; $i++)
			{
				@spelist = split(/~/,$spe{$fld[$i]});
				for ($j = 0; $j <= $#spelist; $j++)
				{
					if ($clade1 =~ /$spelist[$j]/)
					{
						$flag1 = 1;
					}
					elsif ($clade2 =~ /$spelist[$j]/)
					{
						$flag2 = 1;
					}
					elsif ($outgp =~ /$spelist[$j]/)
					{
						$flag3 = 1;
					}
				}
			}
			if ($flag1 + $flag2 + $flag3 >= 2)
			{
				$checkline[$n] = $line;
				$member_num{$checkline[$n]} = $fld[5];
				$n++;
			}
		}
	}
	close(FILE2);

	@sorted_line = sort by_membernum @checkline;
	$MRCA = $loss1 = $loss2 = 0;
	for ($i = 0; $i <= $#sorted_line; $i++)
	{
		@fld = split(" ", $sorted_line[$i]);
		$counter = 0;
		$flag1 = $flag2 = 0;
		$newgene = 0;
		@newly_checked_gene = ();
		for ($j = 6; $j <= $#fld; $j++)
		{
			if ($flag{$fld[$j]} == 1)
			{
				$counter++;
			}
			else
			{
				$newly_checked_gene[$newgene] = $fld[$j];
				$newgene++;
				$flag{$fld[$j]} = 1;
			}
		}
		if ($counter < $member_num{$sorted_line[$i]} && $member_num{$sorted_line[$i]} > 1)
		{
			$flag1 = $flag2 = 0;
			for ($j = 0; $j < $newgene; $j++)
			{
				@spelist = split(/~/,$spe{$newly_checked_gene[$j]});
				for ($k = 0; $k <= $#spelist; $k++)
				{
					if ($clade1 =~ /$spelist[$k]/)
					{
						$flag1 = 1;
					}
					elsif ($clade2 =~ /$spelist[$k]/)
					{
						$flag2 = 1;
					}
				}
			}
			if ($flag1 == 1 && $flag2 == 1)
			#Newly checked genes include both clade 1 and clade 2 genes#
			{
				$MRCA++;
			}
			elsif ($flag1 == 1 && $flag2 == 0)
			#Newly checked genes include clade 1 genes only#
			{	
				$MRCA++;
				$loss2++;
			}
			elsif ($flag1 == 0 && $flag2 == 1)
			#Newly checked genes include clade 2 genes only#
			{
				$MRCA++;
				$loss1++;
			}
			
		}
		$checker = 0;
		foreach $gene(keys(%flag))
		{
			if ($speline =~ /$spe{$gene}/ && $flag{$gene} == 0)
			{
				$checker = 1;
			}
		}
		if ($checker == 0)
		{
			last;
		}
	}
	$line[$l] = $speline;
	$MRCA[$l] = $MRCA;
	$loss1[$l] = $loss1;
	$loss2[$l] = $loss2;
	$clade1[$l] = $clade1;
	$clade2[$l] = $clade2;
	$l++;
}
$linenum = $l;

sub by_membernum
{
	$member_num{$a} <=> $member_num{$b};
}

for ($i = 0; $i < $linenum; $i++)
{
	for ($j = 0; $j < $linenum; $j++)
	{
		if ($clade1[$i] eq $clade1[$j] . "~" . $clade2[$j])
		{
			$gain1[$i] = $MRCA[$j] - ($MRCA[$i] - $loss1[$i]);
		}
		elsif ($clade2[$i] eq $clade1[$j] . "~" . $clade2[$j])
		{
			$gain2[$i] = $MRCA[$j] - ($MRCA[$i] - $loss2[$i]);
		}
	}
	foreach $spe(@spe_list)
	{
		if ($clade1[$i] eq $spe)
		{
			$gain1[$i] = $num{$spe} - ($MRCA[$i] - $loss1[$i]);
		}
		elsif ($clade2[$i] eq $spe)
		{
			$gain2[$i] = $num{$spe} - ($MRCA[$i] - $loss2[$i]);
		}
	}
	printf "%-45s  %3d  %3d / %3d  %3d / %3d\n", $line[$i], $MRCA[$i],
		$gain1[$i], -$loss1[$i], $gain2[$i], -$loss2[$i];
}
foreach $spe(@spe_list)
{
	printf "%-5s %3d\n", $spe, $num{$spe};
}
